package com.theitbulls.contentproviderex;

import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import androidx.annotation.NonNull;
import androidx.appcompat.widget.AppCompatTextView;
import androidx.recyclerview.widget.RecyclerView;

import com.theitbulls.contentproviderex.room.Employee;

import java.util.List;

public class EmployeeListAdapter extends RecyclerView.Adapter<RecyclerView.ViewHolder> {
    private MainActivity activity;
    private final List<Employee> items;

    public EmployeeListAdapter(MainActivity activity, List<Employee> items) {
        this.activity = activity;

        this.items = items;
    }

    @NonNull
    @Override
    public RecyclerView.ViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        final View itemView = LayoutInflater.from(parent.getContext()) .inflate(R.layout.recycler_view_item, parent, false);
        return new EmployeeViewHolder(itemView);
    }

    @Override
    public void onBindViewHolder(@NonNull RecyclerView.ViewHolder holder, int position) {
        final EmployeeViewHolder empHolder = (EmployeeViewHolder) holder;
        final Employee employee = items.get(position);

        empHolder.tvRowId.setText(String.valueOf(employee.id) + ": ");

        empHolder.tvEmpId.setText(employee.empId);
        empHolder.tvEmpName.setText(employee.name);
        empHolder.tvEmpSalary.setText(String.valueOf(employee.salary));
        empHolder.tvEmpType.setText(String.valueOf(employee.type));

        empHolder.btnEdit.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                activity.edit(employee);
            }
        });

        empHolder.btnDelete.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                activity.delete(employee.id);
            }
        });
    }

    @Override
    public int getItemCount() {
        return items.size();
    }

    public class EmployeeViewHolder extends RecyclerView.ViewHolder {
        private final AppCompatTextView tvRowId;

        public AppCompatTextView tvEmpId;
        public AppCompatTextView tvEmpName;
        public AppCompatTextView tvEmpSalary;
        public AppCompatTextView tvEmpType;

        public AppCompatTextView btnEdit;
        public AppCompatTextView btnDelete;

        public EmployeeViewHolder(@NonNull View itemView) {
            super(itemView);

            tvRowId = itemView.findViewById(R.id.empRowId);

            tvEmpId = itemView.findViewById(R.id.empId);
            tvEmpName = itemView.findViewById(R.id.empName);
            tvEmpSalary = itemView.findViewById(R.id.empSalary);
            tvEmpType = itemView.findViewById(R.id.empType);

            btnEdit = itemView.findViewById(R.id.btnEdit);
            btnDelete = itemView.findViewById(R.id.btnDelete);
        }
    }
}
