package com.theitbulls.contentproviderex;

import android.content.ContentProvider;
import android.content.ContentUris;
import android.content.ContentValues;
import android.content.Context;
import android.content.UriMatcher;
import android.database.Cursor;
import android.net.Uri;

import com.theitbulls.contentproviderex.room.AppDatabase;
import com.theitbulls.contentproviderex.room.Employee;
import com.theitbulls.contentproviderex.room.EmployeeDAO;

import org.jetbrains.annotations.NotNull;

public class EmployeeProvider extends ContentProvider {
    private static final String AUTHORITY = "com.theitbulls.contentproviderex";
    public static final String DATA_TYPE = "employee";
    public static final String URL = "content://" + AUTHORITY + "/" + DATA_TYPE;
    public static final Uri CONTENT_URI = Uri.parse(URL);

    private static final UriMatcher uriMatcher = new UriMatcher(UriMatcher.NO_MATCH);

    private static final int EMPLOYEE = 1;
    private static final int EMP_ID = 2;

    static {
        uriMatcher.addURI(AUTHORITY, DATA_TYPE, EMPLOYEE);
        uriMatcher.addURI(AUTHORITY, DATA_TYPE + "/#", EMP_ID);
    }

    public EmployeeProvider() {
    }

    @Override
    public int delete(Uri uri, String selection, String[] selectionArgs) {
        switch (uriMatcher.match(uri)) {
            case EMPLOYEE:
                throw new IllegalArgumentException("Invalid URI, cannot update without ID" + uri);
            case EMP_ID:
                Context mContext = getContext();
                if(mContext == null) {
                    return -1;
                }

                long rowId = ContentUris.parseId(uri);
                return AppDatabase.getInstance(mContext).dao().delete(rowId);
        }
        throw new UnsupportedOperationException("Not yet implemented");
    }

    @Override
    public String getType(@NotNull Uri uri) {
        switch (uriMatcher.match(uri)) {
            case EMPLOYEE:
                return "vnd.android.cursor.dir/" + AUTHORITY + "." + EmployeeDAO.TABLE_NAME;
            case EMP_ID:
                return "vnd.android.cursor.item/" + AUTHORITY + "." + EmployeeDAO.TABLE_NAME;
            default:
                throw new IllegalArgumentException("Unsupported URI: " + uri);
        }
    }

    @Override
    public Uri insert(Uri uri, ContentValues values) {
        switch (uriMatcher.match(uri)) {
            case EMPLOYEE:
                try {
                    Context mContext = getContext();
                    if(mContext == null) {
                        return null;
                    }

                    Employee emp = Employee.getEmployee(values);
                    long rowId = AppDatabase.getInstance(mContext).dao().insert(emp);

                    Uri mUri = ContentUris.withAppendedId(CONTENT_URI, rowId);
                    mContext.getContentResolver().notifyChange(mUri, null);
                    return mUri;
                } finally {
                }
            case EMP_ID:
                throw new IllegalArgumentException("Invalid URI, cannot insert with ID: " + uri);
            default:
                throw new IllegalArgumentException("Unknown URI: " + uri);
        }
    }

    @Override
    public boolean onCreate() {
        return true;
    }

    @Override
    public Cursor query(Uri uri, String[] projection, String selection, String[] selectionArgs, String sortOrder) {
        final int code = uriMatcher.match(uri);
        if (code == EMPLOYEE || code == EMP_ID) {
            final Context context = getContext();
            if (context == null) {
                return null;
            }

            EmployeeDAO dao = AppDatabase.getInstance(context).dao();
            final Cursor cursor;
            if (code == EMPLOYEE) {
                cursor = dao.selectAll();
            } else {
                cursor = dao.selectById(ContentUris.parseId(uri));
            }

            cursor.setNotificationUri(context.getContentResolver(), uri);
            return cursor;

        }
        throw new UnsupportedOperationException("Not yet implemented");
    }

    @Override
    public int update(Uri uri, ContentValues values, String selection, String[] selectionArgs) {
        switch (uriMatcher.match(uri)) {
            case EMPLOYEE:
                throw new IllegalArgumentException("Invalid URI, cannot update without ID" + uri);
            case EMP_ID:
                final Context context = getContext();
                if (context == null) {
                    return 0;
                }

                Employee employee = Employee.getEmployee(values);
                AppDatabase.getInstance(context).dao().insert(employee);
                context.getContentResolver().notifyChange(uri, null);
                return 0;

        }
        throw new UnsupportedOperationException("Not yet implemented");
    }
}
