package com.theitbulls.contentproviderex;

import android.content.ContentValues;
import android.database.Cursor;
import android.net.Uri;
import android.os.AsyncTask;
import android.os.Bundle;
import android.view.View;
import android.widget.Toast;

import androidx.annotation.Nullable;
import androidx.appcompat.app.AppCompatActivity;
import androidx.appcompat.widget.AppCompatButton;
import androidx.appcompat.widget.AppCompatEditText;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.theitbulls.contentproviderex.room.Employee;

import java.util.List;

public class MainActivity extends AppCompatActivity {
    private RecyclerView recyclerView;
    private AppCompatEditText etEmpId;
    private AppCompatEditText etEmpName;
    private AppCompatEditText etEmpSalary;
    private AppCompatEditText etEmpType;

    private boolean toUpdate;
    private Employee employee;

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main_activity);

        etEmpId = findViewById(R.id.empId);
        etEmpName = findViewById(R.id.empName);
        etEmpSalary = findViewById(R.id.empSalary);
        etEmpType = findViewById(R.id.empType);


        LinearLayoutManager llm = new LinearLayoutManager(this);
        llm.setOrientation(LinearLayoutManager.VERTICAL);

        recyclerView = findViewById(R.id.rvEmpList);
        recyclerView.setLayoutManager(llm);

        AppCompatButton btnAdd = findViewById(R.id.btnAdd);
        btnAdd.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {

                try {
                    Employee employee = getEmployee();
                    ContentValues values = Employee.getContentValues(employee);
                    new InsertItemTask().execute(values);
                } catch (Exception e) {
                    Toast.makeText(MainActivity.this, "Record added error: " + e.toString(), Toast.LENGTH_SHORT).show();
                }
            }
        });

        initEmplList();
    }

    private void initEmplList() {
        new ListItemTask().execute();
    }

    public void edit(Employee employee) {
        this.employee = employee;

        etEmpId.setText(employee.empId);
        etEmpName.setText(employee.name);
        etEmpSalary.setText(String.valueOf(employee.salary));
        etEmpType.setText(String.valueOf(employee.type));
    }

    public void delete(long id) {
        String url = EmployeeProvider.URL + "/" + id;
        Uri uri = Uri.parse(url);

        new DeleteItemTask().execute(uri);
    }

    private Employee getEmployee() {
        Employee employee = new Employee();
        employee.empId = etEmpId.getText().toString();
        employee.name = etEmpName.getText().toString();

        String salary = etEmpSalary.getText().toString();
        employee.salary = Double.parseDouble(salary);

        String type = etEmpType.getText().toString();
        employee.type = Integer.parseInt(type);
        return employee;
    }

    public class ListItemTask extends AsyncTask<Void, Void, List<Employee>> {
        @Override
        protected List<Employee> doInBackground(Void... voids) {
            Cursor cursor = getContentResolver().query(EmployeeProvider.CONTENT_URI, null, null, null, null);
            return Employee.getEmployee(cursor);
        }

        @Override
        protected void onPostExecute(final List<Employee> employees) {
            if (employees == null || employees.isEmpty()) {
                Toast.makeText(MainActivity.this, "No record found.", Toast.LENGTH_SHORT).show();
                return;
            }

            runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    EmployeeListAdapter adapter = new EmployeeListAdapter(MainActivity.this, employees);
                    recyclerView.setAdapter(adapter);

                    adapter.notifyDataSetChanged();
                }
            });
        }
    }

    public class InsertItemTask extends AsyncTask<ContentValues, Void, Uri> {
        @Override
        protected Uri doInBackground(ContentValues... values) {
            return getContentResolver().insert(EmployeeProvider.CONTENT_URI, values[0]);
        }

        @Override
        protected void onPostExecute(final Uri uri) {
            if (uri == null) {
                Toast.makeText(MainActivity.this, "Record could not added.", Toast.LENGTH_SHORT).show();
                return;
            }

            initEmplList();
        }
    }

    public class DeleteItemTask extends AsyncTask<Uri, Void, Integer> {
        @Override
        protected Integer doInBackground(Uri... values) {
            return getContentResolver().delete(values[0], null, null);
        }

        @Override
        protected void onPostExecute(Integer integer) {
            if (integer > -1) {
                Toast.makeText(MainActivity.this, "Record deleted successfully.", Toast.LENGTH_SHORT).show();
            }
            initEmplList();
        }
    }

    public class UpdateItemTask extends AsyncTask<ContentValues, Void, Integer> {
        private Uri uri;

        public UpdateItemTask(Uri uri) {
            this.uri = uri;
        }

        @Override
        protected Integer doInBackground(ContentValues... values) {
            return getContentResolver().update(uri, values[0], null, null);
        }

        @Override
        protected void onPostExecute(final Integer count) {
            if (count < 0) {
                Toast.makeText(MainActivity.this, "Record could not updated.", Toast.LENGTH_SHORT).show();
                return;
            }

            employee = null;
            initEmplList();
        }
    }
}
