package com.theitbulls.contentproviderex.room;

import android.content.ContentValues;
import android.database.Cursor;

import androidx.room.ColumnInfo;
import androidx.room.Entity;
import androidx.room.Index;
import androidx.room.PrimaryKey;

import java.util.ArrayList;
import java.util.List;

@Entity(indices = {@Index(value = {Employee.EMP_ID}, unique = true)})
public class Employee {
    public static final String ID = "id";
    public static final String EMP_ID = "emp_id";
    public static final String NAME = "name";
    public static final String SALARY = "salary";
    public static final String TYPE = "type";

    @PrimaryKey(autoGenerate = true)
    public long id;

    @ColumnInfo(name = EMP_ID)
    public String empId;

    @ColumnInfo(name = NAME)
    public String name;

    @ColumnInfo(name = SALARY)
    public double salary;

    @ColumnInfo(name = TYPE)
    public int type;

    public static Employee getEmployee(ContentValues values) {
        Employee emp = new Employee();
        try {
            emp.id = values.getAsLong(Employee.ID);
        } catch (Exception e) {
        }

        emp.empId = values.getAsString(Employee.EMP_ID);
        emp.name = values.getAsString(Employee.NAME);
        emp.salary = values.getAsDouble(Employee.SALARY);
        emp.type = values.getAsInteger(Employee.TYPE);

        return emp;
    }

    public static ContentValues getContentValues(Employee employee) {
        ContentValues values = new ContentValues();
        values.put(ID, employee.id);

        values.put(EMP_ID, employee.empId);
        values.put(NAME, employee.name);
        values.put(SALARY, employee.salary);
        values.put(TYPE, employee.type);

        return values;
    }

    public static List<Employee> getEmployee(Cursor cursor) {
        List<Employee> employees = new ArrayList<>();
        if(cursor != null) {
            while(cursor.moveToNext()) {
                Employee emp = new Employee();
                emp.id = cursor.getLong(cursor.getColumnIndex(ID));

                emp.empId = cursor.getString(cursor.getColumnIndex(EMP_ID));
                emp.name = cursor.getString(cursor.getColumnIndex(NAME));
                emp.salary = cursor.getDouble(cursor.getColumnIndex(SALARY));
                emp.type = cursor.getInt(cursor.getColumnIndex(TYPE));
                employees.add(emp);
            }
        }
        return employees;
    }
}
